@@ -1,5 +1,5 @@
 /* Main handling routines for GNATS.
-   Copyright (C) 1993 Free Software Foundation, Inc.
+   Copyright (C) 1993, 1994, 1995 Free Software Foundation, Inc.
    Contributed by Tim Wicinski (wicinski@barn.com) and
    by Brendan Kehoe (brendan@cygnus.com).
 
@@ -16,42 +16,26 @@ MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License
-along with GNU GNATS; see the file COPYING.  If not, write to
-the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.  */
+along with GNU GNATS; see the file COPYING.  If not, write to the Free
+Software Foundation, 59 Temple Place - Suite 330, Boston, MA 02111, USA.  */
 
 #include "config.h"
-
-#include <stdio.h>
-#include <sys/types.h>
-
-#ifdef HAVE_LIMITS_H
-#include <limits.h>
-#endif
-#ifdef HAVE_SYSLOG_H
-#include <syslog.h>
-#endif
-#include <sys/stat.h>		/* for stat(2) */
-#include <time.h>
-#include <signal.h>
-#include <ctype.h>
-
 #include "gnats.h"
-#include "globals.h"
-#include "pathmax.h"
-#include "headers.h"
-#include "regex.h"
+#include "query.h"
 
 static int verify_analysis	PARAMS((void));
 static void run_atpr		PARAMS((Submitter *, struct tm *, char *, char *));
-static void free_category	PARAMS((Category *));
-static void free_submitter	PARAMS((Submitter *));
-static void free_responsible	PARAMS((Responsible *));
 static void reply_to_submitter	PARAMS((char *));
 static void create_report	PARAMS((char *, int));
 static int get_bug_number	PARAMS((void));
 static void notify_responsible	PARAMS((char*, char*, char*, char*, char*, int, struct tm *));
 
-
+void
+start_data ()
+{
+  return; /* Nothing */
+}
+
 static struct bad_enum *bad_enums = NULL;
 
 void
@@ -61,11 +45,11 @@ gnats (fp)
   int err, bug_number = 0;
   Submitter submitter;
   Category category;
-  char *path = (char *) xmalloc (PATH_MAX);
+  char *path = (char *) alloca (PATH_MAX);
   char number[14]; /* No more than 14 digits in a PR #, ha ha.  */
   char bug_name[STR_MAX];
   Responsible *responsible = NULL;
-  char arrival_time[26];
+  char arrival_time[GNATS_TIME_LENGTH];
   struct tm *expired = NULL;
 
   char *site, *bug_group;
@@ -73,7 +57,7 @@ gnats (fp)
   char *p;
 
   mode_t mode;
-  struct stat buf;
+  struct stat sbuf;
   time_t seconds;
 
   /* Zero things out to make sure.  */
@@ -83,7 +67,7 @@ gnats (fp)
   if (fp == (FILE *) NULL)
     return;
 
-  if (stat (gnats_root, &buf) == -1)
+  if (stat (gnats_root, &sbuf) == -1)
     punt (1, "Can't read gnats_root (looking at %s).", gnats_root);
 
   /* Read the PR in. */
@@ -104,7 +88,7 @@ gnats (fp)
     }
 
   bug_group = field_value (CATEGORY);
-  if (find_categories (&category, bug_group) == -1)
+  if (find_category (&category, bug_group) == -1)
     {
       log_msg (LOG_INFO, 1, "resetting bug category to pending from:",
 	       bug_group);
@@ -113,14 +97,14 @@ gnats (fp)
 
       /* Again, but with `pending' now.  If it's not there, kick this to
 	 the GNATS_ADMIN.  */
-      if (find_categories (&category, bug_group) == -1)
+      if (find_category (&category, bug_group) == -1)
 	punt (1, "No `pending' directory under gnats-root (%s).", gnats_root);
     }
 
   /* Set responsible field, adding full name as well (if found).  If it's
      coming in with a Responsible: field already, don't wipe it out.  */
   p = field_value (RESPONSIBLE);
-  if (p)
+  if (p && p[0])
     {
       char *person, *tmp;
 
@@ -141,6 +125,14 @@ gnats (fp)
 	person = p;
 
       responsible = get_responsible_address (person);
+      if (! responsible)
+	{
+	  log_msg (LOG_INFO, 1, "responsible entry not found for",
+		   person);
+	  responsible = (Responsible *) xmalloc (sizeof (Responsible));
+	  responsible->key = responsible->alias = responsible->fullname
+	    = strdup (person);
+	}
     }
 
   if (! responsible)
@@ -152,16 +144,11 @@ gnats (fp)
 		   category.person);
 	  responsible = get_responsible_address (gnats_admin);
 	  if (responsible == NULL)
-	    punt (0, "can't find gnats-admin in responsibles file. ");
+	    punt (0, "can't find gnats-admin in responsibles file");
 	}
  
-      if (responsible->fullname)
-	sprintf (path, "%s (%s)", responsible->key, responsible->fullname);
-      else
-	sprintf (path, "%s", responsible->key);
-
-      set_field (RESPONSIBLE, path);
-      log_msg (LOG_INFO, 1, "responsible person is:", path);
+      set_field (RESPONSIBLE, responsible->key);
+      log_msg (LOG_INFO, 1, "responsible person is:", responsible->key);
     }
 
   /* Check the subject and the synopsis lines.  If both of these are null,
@@ -193,10 +180,8 @@ gnats (fp)
 
   /* Set arrival date and time of response.  */
   seconds = time ((time_t) 0);
-  strcpy (arrival_time, ctime (&seconds));
-  p = (char *) strrchr (arrival_time, '\n');
-  if (p != NULL)
-    *p = '\0';
+  strftime (arrival_time, GNATS_TIME_LENGTH, "%a %b %e %T %Z %Y",
+	    localtime (&seconds));
   set_field (ARRIVAL_DATE, arrival_time);
 
   /* If the submitter isn't supposed to have a response time, don't bother
@@ -208,7 +193,7 @@ gnats (fp)
      is not there, and the category is PENDING, auto-create that one,
      if we want to.  If not, make the bug pending, and store in there.  */
   sprintf (path, "%s/%s", gnats_root, bug_group);
-  err = stat (path, &buf);
+  err = stat (path, &sbuf);
   if (err == -1 && !flag_autocreate)
     {
       bug_group = PENDING;
@@ -216,7 +201,7 @@ gnats (fp)
       log_msg (LOG_INFO, 1, "directory does not exist, changing to pending:",
 	       path);
       sprintf (path, "%s/%s", gnats_root, bug_group);
-      err = stat (path, &buf);
+      err = stat (path, &sbuf);
     }
 
   /* Check ERR again, to see if pending was there.  */
@@ -291,7 +276,8 @@ gnats (fp)
 
   free_category (&category);
   free_submitter (&submitter);
-/*  free_responsible (responsible); */ /* FIXME */
+  free_responsible (responsible);
+  xfree ((char *)responsible);
 }
 
 static int
@@ -308,7 +294,7 @@ arg_quote (s)
      char *s;
 {
   /* 2x seems a reasonable amount, though 4x is needed for complete safety.  */
-  char *buf = xmalloc (2 * strlen (s));
+  char *buf = xmalloc (2 * (strlen (s) + 1));
   char *p;
 
   for (p = buf; *s; s++, p++)
@@ -336,8 +322,8 @@ run_atpr (submitter, expired, bug_name, path)
      char *bug_name, *path;
 {
   char *at_pr, *command;
-  char buf[STR_MAX];
-  FILE *pipe;
+  char buf[GNATS_TIME_LENGTH];
+  FILE *p;
   int len, i;
   static char *ats[] = { "/usr/bin/at", "/bin/at", NULL };
 
@@ -345,7 +331,7 @@ run_atpr (submitter, expired, bug_name, path)
   strcpy (at_pr, bindir);
   strcat (at_pr, "/at-pr");
 
-  len = strftime (buf, STR_MAX, "%H:%M %b %d", expired);
+  len = strftime (buf, GNATS_TIME_LENGTH, "%H:%M %b %d", expired);
 
   for (i = 0; ats[i]; i++)
     {
@@ -363,8 +349,8 @@ run_atpr (submitter, expired, bug_name, path)
       return;
     }
 
-  pipe = popen (command, "w");
-  if (pipe == (FILE *)NULL)
+  p = popen (command, "w");
+  if (p == (FILE *)NULL)
     {
       punt (0, "Couldn't open the pipe for at-pr.");
       log_msg (LOG_INFO, 0, "popen failed:");
@@ -375,32 +361,30 @@ run_atpr (submitter, expired, bug_name, path)
       char *cont = arg_quote (submitter->contact);
       char *pa = arg_quote (gnats_admin);
       
-      fprintf (pipe, "%s %d %s %s %s '%s' '%s' '%s'\n", at_pr,
+      fprintf (p, "%s %d %s %s %s '%s' '%s' '%s'\n", at_pr,
 	       submitter->rtime, bug_name, path, submitter->key,
 	       orig, cont, pa);
 
-      pclose (pipe);
-      free (orig);
-      free (cont);
-      free (pa);
+      pclose (p);
+      xfree (orig);
+      xfree (cont);
+      xfree (pa);
     }
 }
 
-static char case_fold[256];
-
 /* Check to see if this message is a reply to an existing PR.  If so,
    return the path to that PR.  */
 char *
-check_if_reply (fp)
-    FILE* fp;
+check_if_reply ()
 {
   char token[STR_MAX];
-  char *path = (char *) xmalloc (PATH_MAX);
+  char *path;
   struct stat buf;
-  char *s;
+  char *s, *c;
   struct re_pattern_buffer regex;
   struct re_registers regs;
   int i, start, end;
+  char *case_fold;
 
   s = header_value (SUBJECT);
 
@@ -413,16 +397,13 @@ check_if_reply (fp)
   re_set_syntax (RE_NO_BK_PARENS);
   memset ((void *) &regex, 0, sizeof (regex));
 
-  if (case_fold[1] == 0)
-    {
-      int i;
-      for (i=0; i<256; i++)
-	case_fold[i] = tolower(i);
-    }
+  case_fold = xmalloc (256);
+  for (i=0; i<256; i++)
+    case_fold[i] = tolower(i);
   regex.translate = case_fold;
 
   /* too restrictive? */
-#define PAT "(.*re[ \t]*:)?[ \t]*([-a-zA-Z0-9_+]*/([0-9]+))"
+#define PAT "(.*re[ \t]*(\[[0-9]+\])?:)?[ \t]*([-a-z0-9_+.]*/([0-9]+))"
   re_compile_pattern (PAT, strlen (PAT), &regex);
   i = re_match (&regex, s, strlen (s), 0, &regs);
 
@@ -431,16 +412,27 @@ check_if_reply (fp)
   if (i <= 0)
     return NULL;
 
-  start = regs.start[2];
-  end = regs.end[2];
+  start = regs.start[3];
+  end = regs.end[3];
 
-  free (regs.start);
-  free (regs.end);
+  xfree ((char *)regs.start);
+  xfree ((char *)regs.end);
   
   s[end] = '\0';
-  sprintf (path, "%s/%s", gnats_root, s + start);
+
+  c = strchr (s + start, '/');
+  if (c)
+    path = get_category (c + 1);
+  else
+    {
+      path = xmalloc (PATH_MAX);
+      sprintf (path, "%s/%s", gnats_root, s + start);
+    }
   if (stat (path, &buf) == -1)
-    return NULL;
+    {
+      xfree (path);
+      return NULL;
+    }
 
   return path;
 }
@@ -456,7 +448,7 @@ reply_to_submitter (responsible)
 
   block_signals ();
 
-  msg = open_mail_file (header_value (FROM));
+  msg = open_mail_file ();
   if (msg == (FILE *)NULL)
     punt (1, "%s: cannot open mail agent.", program_name);
 
@@ -477,7 +469,7 @@ reply_to_submitter (responsible)
   fprintf (msg, "Thank you very much for your problem report.\n");
   fprintf (msg, "It has the internal identification `%s/%s'.\n", 
 		field_value (CATEGORY), field_value (NUMBER));
-  fprintf (msg, "The individual assigned to look at your\nbug is: %s. \n",
+  fprintf (msg, "The individual assigned to look at your\nreport is: %s. \n",
 		field_value (RESPONSIBLE));
 
   if (bad_enums != NULL)
@@ -508,13 +500,15 @@ append_notify (person)
     char *person;
 {
   Responsible *responsible;
-  char *notify = (char *) xmalloc (STR_MAX);
+  char *notify, *n;
   int i, notify_len, notify_size = STR_MAX;
   char *p, *end, *start = NULL;
   
   if (person == NULL)
     return NULL;
 
+  notify = xmalloc (STR_MAX);
+
   notify[0] = '\0';
   notify_len = 0;
 
@@ -530,38 +524,32 @@ append_notify (person)
           start = end + 1;
 	}
       else
-	if (start)
-	  *start = '\0';
+	start = NULL;
 
        responsible = get_responsible_address (p);
-       if (responsible != NULL)
+       n = (responsible != NULL && responsible->key != NULL)
+	 ? responsible->alias : p;
+
+       i = strlen (p);
+       /* add in +2 to make checking the second case not a hassle. */
+       if ((i + notify_len + 2) >= notify_size)
 	 {
-	   i = strlen (p);
-	   /* add in +2 to make checking the second case not a hassle. */
-	   if ((i + notify_len + 2) >= notify_size)
-	     {
-	       notify_size += STR_MAX;
-	       notify = (char *) xrealloc (notify, notify_size);
-	     }
-
-	   strcat (notify, responsible->alias);
-	   strcat (notify, ", ");
-	   notify_len += i + 2;
+	   notify_size += STR_MAX;
+	   notify = (char *) xrealloc (notify, notify_size);
 	 }
-      else
-	{
-	  strcat (notify, p);
-	  strcat (notify, ", ");
-	  notify_len += strlen (p) + 2;
-	}
 
-	p = start;
+       strcat (notify, n);
+       strcat (notify, ", ");
+       notify_len += i + 2;
+
+       p = start;
     }
 
   if (*notify)
     return notify;
-  else
-    return NULL;
+
+  xfree (notify);
+  return NULL;
 }
 
 /* Generate a bug report and send it off to the responsible person.  */
@@ -573,10 +561,10 @@ notify_responsible (responsible, subcontact, subnotify, cnotify,
      struct tm *expired;
 {
   FILE *msg;
-  char *notify = (char *) xmalloc (BUFSIZ); /* FIXME - huge */
+  char *notify = (char *) alloca (BUFSIZ); /* FIXME - huge */
   char *subc = NULL, *subn = NULL, *cnote = NULL;
-  char *subject;
-  char buf[STR_MAX];
+  char buf[GNATS_TIME_LENGTH];
+  Header_Name i;
 
   notify[0] = '\0';
 
@@ -584,27 +572,28 @@ notify_responsible (responsible, subcontact, subnotify, cnotify,
   if (subc)
     {
       strcat (notify, subc);
-      free (subc);
+      xfree (subc);
     }
   subn = append_notify (subnotify);
   if (subn)
     {
       strcat (notify, subn);
-      free (subn);
+      xfree (subn);
     }
   cnote = append_notify (cnotify);
   if (cnote)
     {
       strcat (notify, cnote);
-      free (cnote);
+      xfree (cnote);
     }
 
   block_signals ();
 
-  msg = open_mail_file (header_value (FROM));
+  msg = open_mail_file ();
   if (msg == (FILE *)NULL)
     punt (1, "can not open mail agent. ");
 
+#ifdef DONT_USE_RESENT
   {
     char *f = header_value (FROM);
     if (*f)
@@ -615,27 +604,45 @@ notify_responsible (responsible, subcontact, subnotify, cnotify,
   }
 
   fprintf (msg, "To: %s\n", responsible);
-/*   fprintf (msg, "Resent-From: gnats\n"); */
   if (*notify)
-    {
-      fprintf (msg, "Cc: %s\n", notify);
-      free (notify);
-    }
-
-  subject = header_value (SUBJECT);
-  fprintf (msg, "Subject: %s/%s: %s",
-	   field_value (CATEGORY), field_value (NUMBER),
-	   *subject ? subject : "\n");
+    fprintf (msg, "Cc: %s\n", notify);
 
   if (*header_value (DATE))
     fprintf (msg, "In-Reply-To: Your message of %s\t%s",
 	     header_value (DATE), header_value (MSG_ID));
 
+#else /* ! DONT_USE_RESENT */
+  fprintf(msg, "Resent-From: %s (GNATS Management)\n", gnats_user);
+  fprintf(msg, "Resent-To: %s\n", responsible);
+  if (*notify)
+    fprintf(msg, "Resent-Cc: %s\n", notify);
+  {
+    char * reply_to = header_value (REPLY_TO);
+    if (! (reply_to && *reply_to))
+      reply_to = header_value (FROM);
+    fprintf(msg, "Resent-Reply-To: %s, %s", gnats_addr, reply_to);
+  }
+  for (i = RETURN_PATH; i < NUM_HEADER_ITEMS; i++)
+    {
+      if (i == SUBJECT)
+	continue;
+      if (*header_value(i))
+	fprintf(msg, "%s%s", header_name(i), header_value(i));
+    }
+#endif /* DONT_USE_RESENT */
+
+  {
+    char *subject = header_value (SUBJECT);
+    fprintf (msg, "Subject: %s/%s: %s",
+	     field_value (CATEGORY), field_value (NUMBER),
+	     *subject ? subject : "\n");
+  }
+
   fprintf (msg, "\n");
 
   if (flag_notify && rtime != -1 && expired && verify_analysis ())
     {
-      int len = strftime (buf, STR_MAX, "%a %b %d %X %Z %Y", expired);
+      int len = strftime (buf, GNATS_TIME_LENGTH, "%a %b %d %X %Z %Y", expired);
       fprintf (msg,
 	       "\n\tThe contract type is `%s' with a response time of %d business hours.\n",
 	       type, rtime);
@@ -660,7 +667,7 @@ notify_responsible (responsible, subcontact, subnotify, cnotify,
 }
 
 /*  Write a file out to FILENAME with all of the GNATS info in it.  If
-    FORCE is true, then make sure the file doesn't already exist.  */
+    FORCE is false, then make sure the file doesn't already exist.  */
 static void
 create_report (filename, force)
      char *filename;
@@ -702,34 +709,38 @@ append_report (infile, filename)
      char *filename;
 {
   FILE *pr_file, *msg;
-  char line[STR_MAX];
+  char *line = (char *) xmalloc (STR_MAX);
   char *buf, *b;
   int buf_len, buf_max, l;
   Responsible *responsible;
   Submitter submitter;
   char *p, *q;
-  char *from, *to, *subject, *date;
+  char *from, *to, *cc = " ", *subject = " ", *date;
+  time_t t;
 
   /* Save the values we read while in main().  */
   from = (char *) strdup (header_value (FROM));
   to = (char *) strdup (header_value (TO));
-  /* A `Subject:' header is optional.  */
+  /* The `Cc:', `Subject:', and `Date:' headers are optional.  */
+  if (*header_value (CC))
+    cc = (char *) strdup (header_value (CC));
   if (*header_value (SUBJECT))
     subject = (char *) strdup (header_value (SUBJECT));
   if (*header_value (DATE))
     date = (char *) strdup (header_value (DATE));
   else
     {
-      time_t t = time(0);
-      date = (char *) xmalloc (30);
-      strftime (date, 30, "%a, %d %b %y %T %Z\n", localtime(&t));
+      t = time((time_t) 0);
+      date = (char *) xmalloc (GNATS_TIME_LENGTH);
+      strftime (date, GNATS_TIME_LENGTH, "%a, %d %b %y %T %Z\n", localtime(&t));
     }
 
   /* Now read in the PR itself.  */
   pr_file = fopen (filename, "r+");
   if (pr_file == NULL)
     punt (1, "Can't open PR %s.", filename);
-  read_header (pr_file);
+  if (read_header (pr_file) < 0)
+    punt (1, "Error reading PR %s.", filename);
   read_pr (pr_file, 0);
   fclose (pr_file);
 
@@ -746,6 +757,8 @@ append_report (infile, filename)
   APPEND_STRING (line, buf, b, buf_len, buf_max, l, 0);
   sprintf (line, "To: %s", to);
   APPEND_STRING (line, buf, b, buf_len, buf_max, l, 0);
+  sprintf (line, "Cc: %s", cc);
+  APPEND_STRING (line, buf, b, buf_len, buf_max, l, 0);
   sprintf (line, "Subject: %s", subject);
   APPEND_STRING (line, buf, b, buf_len, buf_max, l, 0);
   sprintf (line, "Date: %s", date);
@@ -760,15 +773,25 @@ append_report (infile, filename)
     }
   buf[buf_len] = '\0';
   fclose (infile);
+  xfree (line);
 
   block_signals ();
   /* Create one big thing for the Audit-Trail.  */
   l = strlen (field_value (AUDIT_TRAIL));
-  b = (char *) xrealloc (field_value (AUDIT_TRAIL), l + buf_len + 1);
+  b = (char *) xmalloc (l + buf_len + 1);
+  memcpy (b, field_value (AUDIT_TRAIL), l);
   memcpy (b + l, buf, buf_len);
   b[l + buf_len] = '\0';
   set_field (AUDIT_TRAIL, b);
-  free (b); /* set_field uses strdup() */
+  xfree (b); /* set_field uses strdup() */
+
+  /* Set the >Last-Modified: field.  */
+  t = time ((time_t) 0);
+  xfree (date);
+  date = (char *) xmalloc (GNATS_TIME_LENGTH);
+  strftime (date, GNATS_TIME_LENGTH, "%a %b %e %T %Z %Y", localtime (&t));
+  xfree (pr[LAST_MODIFIED].value);
+  pr[LAST_MODIFIED].value = date;
 
   /* Now write out the PR, with that new field.  */
   create_report (filename, 1);
@@ -780,8 +803,11 @@ append_report (infile, filename)
     *q = '\0';
 
   responsible = get_responsible_address (p);
+  if (! responsible)
+    punt (1, "%s: append_report couldn't look up address %s",
+	  program_name, p);
 
-  msg = open_mail_file (responsible->alias);
+  msg = open_mail_file ();
   if (msg == (FILE *) NULL)
     punt (1, "%s: cannot open mail agent.", program_name);
   fprintf (msg, "To: %s\n", responsible->alias);
@@ -792,24 +818,23 @@ append_report (infile, filename)
   else
     {
       char *addr;
-      char *cc_buf, *cc;
+      char *cc_buf, *new_cc;
       int cc_len, cc_max;
       
-      cc = cc_buf = (char *) xmalloc (BUFSIZ);
+      new_cc = cc_buf = (char *) xmalloc (BUFSIZ);
       cc_len = 0;
       cc_max = BUFSIZ;
       
-      *cc = '\0';
+      *new_cc = '\0';
       addr = append_notify (submitter.contact);
-      APPEND_STRING (addr, cc_buf, cc, cc_len, cc_max, l, 1);
+      APPEND_STRING (addr, cc_buf, new_cc, cc_len, cc_max, l, 1);
       cc_buf[cc_len] = '\0';
 
       fprintf (msg, "Cc: %s\n", cc_buf);
-      free (cc_buf);
+      xfree (cc_buf);
     }
   
   fprintf (msg, "From: %s", from);
-/*   fprintf (msg, "Resent-From: gnats\n"); */
   fprintf (msg, "Subject: %s", subject);
   fprintf (msg, "Reply-To: %s", from);
   fprintf (msg, "\n");
@@ -823,11 +848,11 @@ append_report (infile, filename)
       punt (1, "%s: failed sending mail about a reply to %s\n",
 	    program_name, filename);
     }
-  free (buf);
-  free (from);
-  free (to);
-  free (subject);
-  free (date);
+  xfree (buf);
+  xfree (from);
+  xfree (to);
+  xfree (subject);
+  xfree (date);
 
   close_mail_file (msg);
   unblock_signals ();
@@ -874,49 +899,3 @@ get_bug_number ()
 
   return bug_number;
 }
-
-/* Avoid memory leaks.  */
-static void
-free_submitter (submitter)
-     Submitter *submitter;
-{
-  if (submitter->key)
-    free (submitter->key);
-  if (submitter->fullname)
-    free (submitter->fullname);
-  if (submitter->type)
-    free (submitter->type);
-  if (submitter->contact && submitter->contact != gnats_admin)
-    free (submitter->contact);
-  if (submitter->notify && submitter->notify != gnats_admin)
-    free (submitter->notify);
-  return;
-}
-
-static void
-free_category (category)
-     Category *category;
-{
-  if (category->key)
-    free (category->key);
-  if (category->fullname)
-    free (category->fullname);
-  if (category->person)
-    free (category->person);
-  if (category->notify && category->notify != gnats_admin)
-    free (category->notify);
-  return;
-}
-
-static void
-free_responsible (responsible)
-     Responsible *responsible;
-{
-  if (responsible->key && responsible->key != gnats_admin)
-    free (responsible->key);
-  if (responsible->fullname)
-    free (responsible->fullname);
-  if (responsible->alias && responsible->key != gnats_admin)
-    free (responsible->alias);
-  return;
-}
