@@ -1,5 +1,5 @@
 /* Read in and grok the various administrative files.
-   Copyright (C) 1993 Free Software Foundation, Inc.
+   Copyright (C) 1993, 1994, 1995 Free Software Foundation, Inc.
    Contributed by Tim Wicinski (wicinski@barn.com).
 
 This file is part of GNU GNATS.
@@ -15,27 +15,15 @@ MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License
-along with GNU GNATS; see the file COPYING.  If not, write to
-the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.  */
+along with GNU GNATS; see the file COPYING.  If not, write to the Free
+Software Foundation, 59 Temple Place - Suite 330, Boston, MA 02111, USA.  */
 
 #include "config.h"
-
-#include <stdio.h>
-#include <sys/types.h>
-
-#ifdef HAVE_SYSLOG_H
-#include <syslog.h>
-#endif
-#include <pwd.h>
-
 #include "gnats.h"
-#include "pathmax.h"
-#include "globals.h"
 
 Responsible *responsible_chain;
 
 static int get_adm_record		PARAMS((char*, char**, char*));
-static Responsible *find_responsible	PARAMS((char *));
 static int next_record			PARAMS((char*, char**));
 
 /* Find SITE in the submitters list, copying it into CPTR.  */
@@ -44,8 +32,8 @@ find_submitter (cptr, site)
      Submitter *cptr;
      char *site;
 {
-  char *path = (char *) xmalloc (PATH_MAX);
-  char **array = (char **) xmalloc (NUM_SUBMITTER_FIELDS * sizeof (char *));
+  char *path = (char *) alloca (PATH_MAX);
+  char **array = (char **) alloca (NUM_SUBMITTER_FIELDS * sizeof (char *));
   int err;
 
   if (site == NULL)
@@ -69,6 +57,7 @@ find_submitter (cptr, site)
 	cptr->rtime = atoi (array[3]);
       else
 	cptr->rtime = -1;	/* -1 => no response time specified */
+      xfree (array[3]);
     }
   if (err >= 2)
     cptr->type = array[2];
@@ -88,12 +77,12 @@ find_submitter (cptr, site)
 }
 
 int 
-find_categories (cptr, category)
+find_category (cptr, category)
      Category *cptr;
      char *category;
 {
-  char *path = (char *) xmalloc (PATH_MAX);
-  char **array = (char **) xmalloc (NUM_CATEGORY_FIELDS * sizeof (char *));
+  char *path = (char *) alloca (PATH_MAX);
+  char **array = (char **) alloca (NUM_CATEGORY_FIELDS * sizeof (char *));
   int err;
 
   sprintf (path, "%s/gnats-adm/%s", gnats_root, CATEGORIES);
@@ -101,6 +90,8 @@ find_categories (cptr, category)
 
   if (err == -1)
     return err;
+
+  memset (cptr, 0, sizeof (Category));
   
   if (err >= 3)
     cptr->notify = array[3];
@@ -114,14 +105,16 @@ find_categories (cptr, category)
   return err;
 }
 
-static Responsible*
-find_responsible (person)
+/* Note: use get_responsible_address, not this!  */
+static int
+find_responsible (responsible, person)
+     Responsible *responsible;
      char *person;
 {
   Responsible *r;
 
-  if (person == NULL)
-    return NULL;
+  if (person == NULL || person[0] == '\0')
+    return -1;
 
   /* If the chain has worked, then search it. Otherwise, do the 'file
      thing'. */
@@ -129,7 +122,12 @@ find_responsible (person)
     {
       for (r = responsible_chain; r; r = r->next)
         if (strcmp (r->key, person) == 0)
-          return r;
+	  {
+	    responsible->key = r->key;
+	    responsible->fullname = r->fullname;
+	    responsible->alias = r->alias;
+	    return 0;
+	  }
     }
   else
     {
@@ -139,21 +137,22 @@ find_responsible (person)
 
       sprintf (path, "%s/gnats-adm/%s", gnats_root, RESPONSIBLE_FILE);
       err = get_adm_record (path, array, person);
-
+      xfree (path);
       if (err == -1)
-        return NULL;
+	{
+	  xfree ((char *) array);
+	  return err;
+	}
 
-      r = (Responsible *) xmalloc (sizeof (Responsible));
-      memset (r, 0, sizeof (Responsible));
+      responsible->alias = array[2];
+      responsible->fullname = array[1];
+      responsible->key = array[0];
 
-      r->alias = array[2];
-      r->fullname = array[1];
-      r->key = array[0];
-
-      return r;
+      xfree ((char *) array);
+      return 0;
     }
 
-  return NULL;
+  return -1;
 }
 
 /* Read through the adm files, finding the line that matches the
@@ -167,13 +166,13 @@ get_adm_record (fname, ptr, key)
 {
   FILE *fp;
   char line[STR_MAX], temp[STR_MAX];
-  char *l;
+  char *l, *l2;
   char *keyed;
-  int err = -1, i, keyedlen;
+  int err = -1, keyedlen;
 
   /* append a delimiting ':' to the end of sting to make sure the match
      is fully correct.  */
-  keyed = (char *) xmalloc (strlen(key) + 2);
+  keyed = (char *) alloca (strlen(key) + 2);
   /* which is slower - strcpy/strcat or sprintf? */
   sprintf (keyed, "%s:", key);
   keyedlen = strlen(keyed); 
@@ -183,28 +182,15 @@ get_adm_record (fname, ptr, key)
     return -1;
 
   while (read_string (line, fp) > 0)
-    if (line[0] != '#' && line[0] != ' ' && line[0] != '\n') 
+    if (line[0] != '#' && line[0] != ' ' && line[0] != '\n' &&
+	!strncasecmp(line, keyed, keyedlen))
       {
-        i = strncmp (line, keyed, keyedlen);
-        if (i == 0)
-          { 
-            /* We found a match, now get the whole line.  */
-            l = line;
-            while (*l != '\0') 
-              {
-                l = get_next_field (l, temp, ':');
-                if (temp != NULL)
-		  {
-		    /* We got something.  */
-		    i = strlen (temp);
-		    if (temp[i - 1] == '\n')
-		      temp[i - 1] = '\0';
-		    ptr[++err] = (char *) strdup (temp);
-		    temp[0] = '\0';
-		  }
-	      }
-	    break;
-          }
+	/* We found a match, now get the whole line.  */
+	for (l2 = line, l = get_next_field(l2, temp, ':');
+	     l || l2;
+	     (l2 = l) && (l = get_next_field(l, temp, ':')))
+	  ptr[++err] = (char *) strdup (temp);
+	break;
       }
 
   fclose (fp);
@@ -219,29 +205,40 @@ Responsible *
 get_responsible_address (person)
      char *person;
 {
-  Responsible* responsible = NULL;
+  Responsible* responsible = (Responsible *) xmalloc (sizeof (Responsible));
   struct passwd *passwd;
   char *p;
 
   /* First check the responsible file; if there's no entry, try the
      passwd file.  */
-  responsible = find_responsible (person);
-  if (responsible)
+  if (find_responsible (responsible, person) != -1)
     {
-      if (responsible->alias[0] == '\0')
-	responsible->alias = responsible->key;
+      if (!responsible->alias || responsible->alias[0] == '\0')
+	{
+	  xfree (responsible->alias);
+	  responsible->alias = strdup (responsible->key);
+	}
     }
-  else if ((passwd = getpwnam (person)) != 0)
+  else
     {
-      responsible = (Responsible*) xmalloc (sizeof (Responsible));
-      responsible->key = responsible->alias = (char *) strdup (person);
+      /* We should always allow names to show up as responsible, even
+	 if they aren't listed in the passwd file---folks don't remember
+	 (or usually need) to change closed PRs if the listed person
+	 happens to leave the company.  */
+      responsible->key = (char *) strdup (person);
+      responsible->alias = (char *) strdup (responsible->key);
 
-      /* Some passwd entries have commas for finger programs that
-	 understand office phone numbers, etc.  Chop 'em off.  */
-      p = (char *) strchr (passwd->pw_gecos, ',');
-      if (p != NULL)
-	*p = '\0';
-      responsible->fullname = (char *) strdup (passwd->pw_gecos);
+      if ((passwd = getpwnam (person)) != 0)
+	{
+	  /* Some passwd entries have commas for finger programs that
+	     understand office phone numbers, etc.  Chop 'em off.  */
+	  p = (char *) strchr (passwd->pw_gecos, ',');
+	  if (p != NULL)
+	    *p = '\0';
+	  responsible->fullname = (char *) strdup (passwd->pw_gecos);
+	}
+      else
+	responsible->fullname = (char*) NULL;
     } 
 
   return responsible;
@@ -258,9 +255,10 @@ init_responsibles ()
   FILE *fp;
 
   char line[STR_MAX];
-  char **array = (char **) xmalloc (NUM_RESPONSIBLE_FIELDS * sizeof (char *));
-  char *path = xmalloc (PATH_MAX);
+  char **array = (char **) alloca (NUM_RESPONSIBLE_FIELDS * sizeof (char *));
+  char *path = (char *) alloca (PATH_MAX);
 
+  memset (array, 0, NUM_RESPONSIBLE_FIELDS * sizeof (char *));
   sprintf (path, "%s/gnats-adm/%s", gnats_root, RESPONSIBLE_FILE);
   fp = fopen (path, "r");
   if (fp == NULL)
@@ -269,14 +267,26 @@ init_responsibles ()
   while (read_string (line, fp) > 0)
     if (line[0] != '#' && line[0] != ' ' && line[0] != '\n')
       {
-	next_record (line, array);
+	int fields;
+
+	fields = next_record (line, array);
 	r = (Responsible *) xmalloc (sizeof (Responsible));
 	r->key = array[0];
-	r->fullname = array[1];
-	r->alias = array[2];
-
-	if (!*r->alias)
-          r->alias = r->key;
+	if (fields > 1)
+	  r->fullname = array[1];
+	else
+	  r->fullname = strdup ("");
+	if (fields > 2)
+	  {
+	    r->alias = array[2];
+	    if (!*r->alias)
+	      {
+		xfree (r->alias);
+		r->alias = r->key;
+	      }
+	  }
+	else
+	  r->alias = r->key;
 
 	if (r_end == NULL)
 	  r_start = r_end = r;
@@ -308,24 +318,59 @@ next_record (line, ptr)
    char *line;
    char **ptr;
 {
-  char *l;
-  int i, err = 0;
+  char *l, *l2;
+  int err = 0;
   char temp[STR_MAX];
 
-  l = line;
-  while (*l != '\0')
-    {
-      l = get_next_field (l, temp, ':');
-      if (temp != NULL)
-        {
-          /* We got something.  */
-          i = strlen (temp);
-          if (temp[i - 1] == '\n')
-            temp[i - 1] = '\0';
-          ptr[err++] = (char *) strdup (temp);
-          temp[0] = '\0';
-        }
-    }
+  for (l2 = line, l = get_next_field(l2, temp, ':');
+       l || l2;
+       (l2 = l) && (l = get_next_field(l, temp, ':')))
+    ptr[err++] = (char *) strdup (temp);
 
   return err;
 }
+
+void
+free_responsible (responsible)
+     Responsible *responsible;
+{
+  if (responsible->key && responsible->key != gnats_admin)
+    xfree (responsible->key);
+  if (responsible->fullname)
+    xfree (responsible->fullname);
+  if (responsible->alias && responsible->key != gnats_admin)
+    xfree (responsible->alias);
+  return;
+}
+
+void
+free_submitter (submitter)
+     Submitter *submitter;
+{
+  if (submitter->key)
+    xfree (submitter->key);
+  if (submitter->fullname)
+    xfree (submitter->fullname);
+  if (submitter->type)
+    xfree (submitter->type);
+  if (submitter->contact && submitter->contact != gnats_admin)
+    xfree (submitter->contact);
+  if (submitter->notify && submitter->notify != gnats_admin)
+    xfree (submitter->notify);
+  return;
+}
+
+void
+free_category (category)
+     Category *category;
+{
+  if (category->key)
+    xfree (category->key);
+  if (category->fullname)
+    xfree (category->fullname);
+  if (category->person)
+    xfree (category->person);
+  if (category->notify && category->notify != gnats_admin)
+    xfree (category->notify);
+  return;
+}
