@@ -1,6 +1,7 @@
 /* Miscellaneous utility routines for GNATS.
-   Copyright (C) 1993 Free Software Foundation, Inc.
-   Contributed by Tim Wicinski (wicinski@barn.com).
+   Copyright (C) 1993, 1994, 1995 Free Software Foundation, Inc.
+   Contributed by Tim Wicinski (wicinski@barn.com)
+   and Brendan Kehoe (brendan@cygnus.com).
 
 This file is part of GNU GNATS.
 
@@ -15,31 +16,13 @@ MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License
-along with GNU GNATS; see the file COPYING.  If not, write to
-the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
+along with GNU GNATS; see the file COPYING.  If not, write to the Free
+Software Foundation, 59 Temple Place - Suite 330, Boston, MA 02111, USA.
 */
 
 #include "config.h"
-
-#if defined(__STDC__) || defined(_AIX)
-#include <stdarg.h>
-#define VA_START(args, lastarg) va_start(args, lastarg)
-#else
-#include <varargs.h>
-#define VA_START(args, lastarg) va_start(args)
-#endif
-
-#ifdef HAVE_SYSLOG_H
-#include <syslog.h>
-#endif
-
-#include <signal.h>
-#include <sys/types.h>
-#include <sys/stat.h>
-
 #include "gnats.h"
-#include "globals.h"
-#include "pathmax.h"
+#include "gnatsd.h"
 
 /* take a link list of index entries and write it back out... */
 void
@@ -48,47 +31,77 @@ write_index (index_start)
 {
 
   FILE *fp;
-  char *path = (char *) xmalloc (PATH_MAX);
-  char *tmpfile = (char *) xmalloc (PATH_MAX);
+  char *path, *workfile;
+#ifndef HAVE_MKTEMP
+  char name[L_tmpnam];
+#endif
   Index *i;
 
-  sprintf (tmpfile, "%s/gnats-adm/indXXXXXX", gnats_root);
-  sprintf (path, "%s/gnats-adm/%s", gnats_root, INDEX);
-  mktemp (tmpfile);
+#ifdef HAVE_MKTEMP
+  workfile = (char *) xmalloc (strlen (gnats_root) +
+			       strlen ("/gnats-adm/indXXXXXX") +
+			       1 /* null */ );
+  sprintf (workfile, "%s/gnats-adm/indXXXXXX", gnats_root);
+  mktemp (workfile);
+#else
+  workfile = (char *) xmalloc (L_tmpnam);
+  tmpnam (name);
+  strcpy (workfile, name);
+#endif
 
-  fp = fopen (tmpfile, "w");
+  fp = fopen (workfile, "w");
   if (fp == NULL)
     {
       fprintf (stderr, "%s: can't open the temporary file %s\n",
-               program_name, tmpfile);
+               program_name, workfile);
+      xfree (workfile);
       return;
     }
 
   for (i = index_start; i ; i = i->next) 
     {
+#ifdef GNATS_RELEASE_BASED
+      fprintf(fp, "%s/%s:%s:%s:%s:%s:%s:%s:%d\n", i->category, i->number,
+        i->submitter, i->responsible, i->state, i->confidential,
+        i->severity, i->priority,
+	      (i->date_required && *i->date_required) ? atoi(i->date_required)
+	      : 0);
+#else
       fprintf(fp, "%s/%s:%s:%s:%s:%s:%s:%s\n", i->category, i->number,
         i->submitter, i->responsible, i->state, i->confidential,
         i->severity, i->priority);
+#endif
     }
 
   fclose (fp);
 
   block_signals ();
 
-  if ((rename (tmpfile, path)) < 0)
+  path = (char *) xmalloc (strlen (gnats_root) +
+			   strlen ("/gnats-adm/") +
+			   strlen (INDEX) +
+			   1 /* null*/ );
+  sprintf (path, "%s/gnats-adm/%s", gnats_root, INDEX);
+
+  if ((rename (workfile, path)) < 0)
     {
       if (errno != EXDEV)
-	punt (1, "%s: could not rename temporary index %s into gnats-adm: %s\n",
-	      program_name, tmpfile, strerror (errno));
+	{
+	  xfree (path);
+	  punt (1, "%s: could not rename temporary index %s into gnats-adm: %s\n",
+		program_name, workfile, strerror (errno));
+	}
 
-      if (copy_file (tmpfile, path))
+      if (copy_file (workfile, path))
 	punt (1, "%s: could not copy temporary index %s into %s: %s\n",
-	      program_name, tmpfile, path, strerror (errno));
+	      program_name, workfile, path, strerror (errno));
 
-      unlink (tmpfile);
+      unlink (workfile);
     }
 
-  unblock_signals (); 
+  xfree (path);
+  xfree (workfile);
+  unblock_signals ();
 }
 
 /* Add the current PR to the GNATS index file.  */
@@ -96,8 +109,8 @@ void
 add_to_index ()
 {
   FILE *fp;
-  char *path = (char *) xmalloc (PATH_MAX);
-  char *buf = (char *) xmalloc (STR_MAX);
+  char *path = (char *) alloca (PATH_MAX);
+  char *buf = (char *) alloca (STR_MAX);
 
   block_signals ();
 
@@ -107,6 +120,7 @@ add_to_index ()
   if (fp == NULL) 
     punt (1, "Can't append to the GNATS index file (%s).", path);
 
+  memset ((void *) buf, 0, sizeof (STR_MAX));
   create_index_entry (buf);
   fputs (buf, fp);
 
@@ -195,11 +209,22 @@ copy_file (source, dest)
   return 0;
 }
 
+extern int is_daemon;
+
 /* Try to lock the file noted by FP.  */
-void 
+int
+is_gnats_locked ()
+{
+  char *path = (char *) alloca (PATH_MAX);
+  struct stat buf;
+  sprintf (path, "%s/gnats-adm/gnats.lock", gnats_root);
+  return stat (path, &buf) == 0;
+}
+
+int
 lock_gnats ()
 {
-  char *path = (char *) xmalloc (PATH_MAX);
+  char *path = (char *) alloca (PATH_MAX);
   struct stat buf;
   int count;
 
@@ -212,37 +237,75 @@ lock_gnats ()
        count < MAXWAIT / GRANULARITY && stat (path, &buf) != -1 ;
        sleep(GRANULARITY), ++count)
     ;
-  if (stat(path, &buf) != -1)
+  if (stat (path, &buf) != -1)
     {
-      fprintf (stderr, "lock file exists %s\n", path);
-      exit (1);
+      if (is_daemon)
+	{
+	  printf ("%d GNATS lock file exists.\r\n", CODE_GNATS_LOCKED);
+	  return -1;
+	}
+      else
+	{
+	  fprintf (stderr, "%s: GNATS lock file exists %s\n",
+		   program_name, path);
+	  exit (1);
+	}
     }
 
-  if (creat (path, 0) == -1)
+  if (creat (path, 0) < 0)
     {
-      fprintf (stderr, "can not create lock file: %s\n", path);
-      exit (3);
+      if (is_daemon)
+	{
+	  printf ("%d GNATS server cannot create lock file: %s\r\n",
+		  CODE_ERROR, strerror (errno));
+	  return -1;
+	}
+      else
+	{
+	  fprintf (stderr, "%s: cannot create lock file: %s\n",
+		   program_name, strerror (errno));
+	  exit (3);
+	}
     }
+
+  if (is_daemon)
+    printf ("%d GNATS database is now locked.\r\n", CODE_OK);
+  return 0;
 }
 
-void 
+void
 unlock_gnats ()
 {
-  char *path = (char *) xmalloc (PATH_MAX);
+  char *path = (char *) alloca (PATH_MAX);
   struct stat buf;
 
   sprintf (path, "%s/gnats-adm/gnats.lock", gnats_root);
   
   if (stat (path, &buf) < 0)
-    return;
+    {
+      /* We're very quite about this.  */
+      if (is_daemon)
+	printf ("%d GNATS database is already unlocked.\r\n", CODE_OK);
+      return;
+    }
 
   if (unlink (path) < 0)
     {
-      fprintf (stderr, "%s: cannot unlink lock file %s: %s\n",
-	       program_name, path, (char *) strerror (errno));
-      exit (3);
+      if (is_daemon)
+	{
+	  printf ("%d GNATS is not locked.\r\n", CODE_FILE_ERROR);
+	  return;
+	}
+      else
+	{
+	  fprintf (stderr, "%s: cannot unlink lock file %s: %s\n",
+		   program_name, path, (char *) strerror (errno));
+	  exit (3);
+	}
     }
 
+  if (is_daemon)
+    printf ("%d GNATS database unlocked.\r\n", CODE_OK);
 }
 
 int
@@ -277,7 +340,7 @@ punt (die, va_alist)
 
   fmt = va_arg (args, char *);
   
-  fp = open_mail_file (gnats_admin);
+  fp = open_mail_file ();
   if (fp == (FILE *) NULL)
     {
       fprintf (stderr, "can not open mail file for errors...panic!");
@@ -285,10 +348,8 @@ punt (die, va_alist)
       exit (3);
     }
 
-  fprintf (fp, "From: gnats (GNATS Management)\n");
-#ifndef USE_BIN_MAIL
+  fprintf (fp, "From: %s (GNATS Management)\n", gnats_user);
   fprintf (fp, "To: %s\n", gnats_admin);
-#endif
   fprintf (fp, "Subject: problem with %s\n", program_name);
   fprintf (fp, "\n\n");		/* Two newlines to massage buggy sendmails */
 
@@ -306,7 +367,8 @@ punt (die, va_alist)
 }
 
 void
-unlock_quit ()
+unlock_quit (sig)
+     int sig;
 {
   unlock_gnats ();
   unblock_signals ();
