@@ -1,6 +1,7 @@
 /* Entry for the GNATS system.
-   Copyright (C) 1993 Free Software Foundation, Inc.
-   Contributed by Tim Wicinski (wicinski@barn.com).
+   Copyright (C) 1993, 1994, 1995 Free Software Foundation, Inc.
+   Contributed by Tim Wicinski (wicinski@barn.com)
+   and hacked by Brendan Kehoe (brendan@cygnus.com).
 
 This file is part of GNU GNATS.
 
@@ -15,28 +16,11 @@ MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License
-along with GNU GNATS; see the file COPYING.  If not, write to
-the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.  */
+along with GNU GNATS; see the file COPYING.  If not, write to the Free
+Software Foundation, 59 Temple Place - Suite 330, Boston, MA 02111, USA.  */
 
 #include "config.h"
-
-#include <stdio.h>
-#include <sys/types.h>
-
-#include <getopt.h>
-#include <ansidecl.h>
-#ifdef HAVE_SYSLOG_H
-#include <syslog.h>
-#endif
-
-#include <signal.h>
-
-#ifdef HAVE_UNISTD_H
-#include <unistd.h>
-#endif
-
 #include "gnats.h"
-#include "globals.h"
 
 static void version		PARAMS((void));
 
@@ -56,6 +40,9 @@ int log_level = LOG_LEVEL;
    response time or not.  */
 char *at_pr = NULL;
 
+/* If 1, we're running the daemon.  */
+int is_daemon = 0;
+
 struct option long_options[] =
 {
   {"directory", 1, NULL, 'd'},
@@ -71,7 +58,8 @@ struct option long_options[] =
 };
 
 void
-unlock_death ()
+unlock_death (sig)
+     int sig;
 {
   signal (SIGSEGV, SIG_DFL);
   signal (SIGHUP, SIG_DFL);
@@ -96,7 +84,7 @@ static char USAGE[] = "\
 Usage: file-pr [-DVh] [-d gnats-root] [-f filename] [--directory=gnats-root]\n\
        [--file=filename] [--debug] [--version] [--help]\n";
 
-void
+int
 main (argc, argv)
      int argc;
      char **argv;
@@ -109,7 +97,6 @@ main (argc, argv)
   char *pr_path;
 
   program_name = basename (argv[0]);
-  configure ();
 
   /* Note: `A' isn't included here yet.  */
   while ((option = getopt_long (argc, argv, "d:f:VDh",
@@ -119,7 +106,6 @@ main (argc, argv)
 	{
 	case 'd':
 	  gnats_root = optarg;
-	  configure ();
 	  break;
 
 	case 'f':
@@ -153,6 +139,8 @@ main (argc, argv)
 	}
     }
 
+  configure ();
+
   /* Check the options after processing.  Currently, check logging options.  */
   if (logging != NULL)
     {
@@ -172,7 +160,7 @@ main (argc, argv)
     {
     case MAIL:
       /* FIXME */
-      gnats_logfile = open_mail_file (gnats_admin);
+      gnats_logfile = open_mail_file ();
       if (gnats_logfile == (FILE *)NULL)
 	punt (1, "can not open mail agent. ");
 
@@ -223,12 +211,13 @@ main (argc, argv)
   lock_gnats ();
 
   /* Read the message header in.  */
-  read_header (fp);
+  if (read_header (fp) < 0)
+    punt (1, "failure reading PR");
 
   /* Check if the message is a reply to an existing PR; if it is, then
      just append this message to the end of that PR.  Otherwise, process
      it normally.  */
-  pr_path = check_if_reply (fp);
+  pr_path = check_if_reply ();
   if (pr_path)
     {
       /* Before inserting the reply, see if someone's got the PR
@@ -241,6 +230,7 @@ main (argc, argv)
 	  exit (1);
 	}
       append_report (fp, pr_path);
+      xfree (pr_path);
     }
   else
     gnats (fp);
