@@ -1,5 +1,5 @@
 /* Handle getting a message from email into GNATS.
-   Copyright (C) 1993 Free Software Foundation, Inc.
+   Copyright (C) 1993, 1994, 1995 Free Software Foundation, Inc.
    Contributed by Brendan Kehoe (brendan@cygnus.com).
 
 This file is part of GNU GNATS.
@@ -15,30 +15,16 @@ MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License
-along with GNU GNATS; see the file COPYING.  If not, write to
-the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.  */
+along with GNU GNATS; see the file COPYING.  If not, write to the Free
+Software Foundation, 59 Temple Place - Suite 330, Boston, MA 02111, USA.  */
 
 #include "config.h"
 
-#include <stdio.h>
-#include <sys/types.h>
-#include <getopt.h>
-
-#ifdef HAVE_SYSLOG_H
-#include <syslog.h>
-#endif
-
-#ifdef HAVE_UNISTD_H
-#include <unistd.h>
-#endif
-
 #include <sys/wait.h> /* FIXME */
 #include <sys/param.h> /* FIXME: gets MAXBSIZE */
 
 #include "gnats-dirs.h"
-#include "globals.h"
 #include "gnats.h"
-#include "pathmax.h"
 
 #ifndef MAXBSIZE /* FIXME - from params.h */
 #define MAXBSIZE 8192
@@ -60,11 +46,11 @@ int flag_debug = 0;
    queue for GNATS.  */
 char *queue_file = NULL;
 
-/* Where the GNATS queue directory is.  Default is GNATS_ROOT/gnats-queue.  */
-char *queue_dir = NULL;
+/* Directory where incoming PRs live */
+char *queue_dir;
 
-/* A safe environment for execle().  */
-static char *safe_env[] = { "USER=gnats", "PATH=/bin:/usr/bin", NULL };
+/* If 1, we're running the daemon.  */
+int is_daemon = 0;
 
 struct option long_options[] =
 {
@@ -84,9 +70,15 @@ int
 fork_gnats (filename)
      char *filename;
 {
+  /* A safe environment for execle().  */
+  static char *safe_env[] = { "", "PATH=/bin:/usr/bin", NULL };
+
   int pid; /* pid_t */
   int status;
 
+  safe_env[0] = (char *) xmalloc (5 + strlen (gnats_user) + 1);
+  sprintf (safe_env[0], "USER=%s", gnats_user);
+
   errno = 0;
   pid = fork();
   if (pid < 0)
@@ -119,13 +111,14 @@ fork_gnats (filename)
       if (flag_debug)
 	{
 	  if (execle (gnats_bin, "file-pr", "-f", filename,
-		      "-D", NULL, safe_env) < 0)
+		      "-D", "-d", gnats_root, NULL, safe_env) < 0)
 	    punt (1, "%s: execle of gnats failed: %s\n", program_name,
 		  strerror (errno));
 	}
       else
 	{
-	  if (execle (gnats_bin, "file-pr", "-f", filename, NULL, safe_env) < 0)
+	  if (execle (gnats_bin, "file-pr", "-f", filename,
+		      "-d", gnats_root, NULL, safe_env) < 0)
 	    punt (1, "%s: execle of gnats failed: %s\n", program_name,
 		  strerror (errno));
 	}
@@ -168,6 +161,12 @@ run_gnats ()
   while ((next = readdir (d)))
     if (next->d_name[0] != '.')
       {
+	if (strcmp (next->d_name, "core") == 0)
+	  {
+	    log_msg (LOG_INFO, 0, "core file in queue directory");
+	    continue;
+	  }
+
 	if (nfiles == maxfiles)
 	  {
 	    maxfiles *= 2;
@@ -177,8 +176,7 @@ run_gnats ()
 	files[nfiles++].name = (char *) strdup (next->d_name);
       }
 
-  if (CLOSEDIR (d))
-    log_msg (LOG_INFO, 1, "can not close: ", queue_dir);
+  closedir (d);
 
   /* Run a gnats process for each file in the directory.  */
   for (i = 0; i < nfiles; i++)
@@ -209,13 +207,12 @@ run_gnats ()
 	  rename (files[i].name, name2);
 	  punt (0, "renamed `%s' to `%s' pending human intervention.\n",
 		files[i].name, name2);
-	  free (name2);
-	  if (gnats_locked ()) unlock_gnats ();
+	  xfree (name2);
+	  if (gnats_locked ())
+	    unlock_gnats ();
 	}
       else if (child_status == 3)
-	{
-	  punt (0, "file-pr complained of a (probably) general problem.\n");
-	}
+	punt (0, "file-pr complained of a (probably) general problem.\n");
     }
 }
 
@@ -230,6 +227,9 @@ drop_msg ()
   int r; /* XXX ssize_t */
   char *buf = (char *) xmalloc (MAXBSIZE);
   char *base, *new_name;
+#ifndef HAVE_MKTEMP
+  char name[L_tmpnam];
+#endif
 
   if (queue_file)
     {
@@ -244,8 +244,13 @@ drop_msg ()
   tmpdir = getenv ("TMPDIR");
   if (tmpdir == NULL)
     tmpdir = "/tmp"; /* FIXME */
+#ifdef HAVE_MKTEMP
   sprintf (bug_file, "%s/gnatsXXXXXX", tmpdir);
   mktemp (bug_file);
+#else
+  tmpnam (name);
+  strcpy (bug_file, name);
+#endif
   
   fd[1] = open (bug_file, O_WRONLY|O_CREAT, 0664);
   if (fd[1] < 0)
@@ -291,10 +296,10 @@ drop_msg ()
 }
 
 static char USAGE[] = "\
-Usage: %s [-f filename] [-d directory] [-q] [-r]\n\
-       [--file=filename] [--directory=directory] [--queue] [--run]\n";
+Usage: %s [-f filename] [-d gnats-root] [-q] [-r]\n\
+       [--file=filename] [--directory=gnats-root] [--queue] [--run]\n";
 
-void
+int
 main (argc, argv)
      int argc;
      char **argv;
@@ -313,7 +318,7 @@ main (argc, argv)
 	case 'd':
 	  if (optarg[0] == '\0')
 	    punt (1, "%s: directory must be non-null\n", program_name);
-	  queue_dir = optarg;
+	  gnats_root = optarg;
 	  break;
 
 	case 'f':
@@ -343,10 +348,12 @@ main (argc, argv)
 	case 'h':
 	  fprintf (stderr, USAGE, program_name);
 	  exit (0);
+	  break;
 
 	case 'V':
 	  fprintf (stderr, "queue-pr version %s.\n", version_string);
 	  exit (0);
+	  break;
 
 	default:
 	  punt (1, USAGE, program_name);
@@ -356,12 +363,16 @@ main (argc, argv)
   configure ();
   umask (022);
 
-  if (! queue_dir)
+  if (! gnats_root)
     {
-      queue_dir = (char *) xmalloc (strlen (gnats_root) + 13);
-      strcpy (queue_dir, gnats_root);
-      strcat (queue_dir, "/gnats-queue");
+      fprintf (stderr, "%s: null value for GNATS root directory\n",
+	       program_name);
+      exit (1);
     }
+  queue_dir = (char *) xmalloc (strlen (gnats_root)
+				+ strlen ("/gnats-queue")
+				+ 1 /* null char */);
+  sprintf (queue_dir, "%s/gnats-queue", gnats_root);
 
   if (queue_msg)
     drop_msg ();
