@@ -2,7 +2,7 @@
 # Submit a problem report to a GNATS site.
 # Copyright (C) 1993 Free Software Foundation, Inc.
 # Contributed by Brendan Kehoe (brendan@cygnus.com), based on a
-# version written by Heinz G. Seidl (hgs@ide.com).
+# version written by Heinz G. Seidl (hgs@cygnus.com).
 #
 # This file is part of GNU GNATS.
 #
@@ -37,7 +37,9 @@ GNATS_ADDR=xGNATS_ADDRx
 DATADIR=xDATADIRx
 
 # If we've been moved around, try using GCC_EXEC_PREFIX.
-[ ! -d $DATADIR/gnats -a -d "$GCC_EXEC_PREFIX" ] && DATADIR=${GCC_EXEC_PREFIX}..
+[ ! -d $DATADIR/gnats -a -d "$GCC_EXEC_PREFIX" ] && 
+  DATADIR=${GCC_EXEC_PREFIX}../../../lib
+
 
 # The default release for this host.
 DEFAULT_RELEASE="xDEFAULT_RELEASEx"
@@ -55,6 +57,9 @@ GNATS_SITE=xGNATS_SITEx
 # host-dependent.
 MAIL_AGENT="xMAIL_AGENTx"
 
+# How to read the passwd database.
+PASSWD="xPASSWDx"
+
 ECHON=xECHONx
 
 if [ $ECHON = bsd ] ; then
@@ -70,37 +75,41 @@ fi
 
 #
 
-[ -z "$TMPDIR" ] && TMPDIR=/tmp
+if [ -z "$TMPDIR" ]; then
+  TMPDIR=/tmp
+else
+  if [ "`echo $TMPDIR | grep '/$'`" != "" ]; then
+    TMPDIR="`echo $TMPDIR | sed -e 's,/$,,'`"
+  fi
+fi
 
 TEMP=$TMPDIR/p$$
 BAD=$TMPDIR/pbad$$
 REF=$TMPDIR/pf$$
 
-if [ -z "$LOGNAME" -a -n "$USER" ]; then
-  LOGNAME=$USER
+# find a user name
+if [ "$LOGNAME" = "" ]; then
+	if [ "$USER" != "" ]; then
+		LOGNAME="$USER"
+	else
+		LOGNAME="UNKNOWN"
+	fi
 fi
 
 FROM="$LOGNAME"
-REPLY_TO="$LOGNAME"
+if [ -z "$REPLYTO" ]; then
+  REPLYTO="$LOGNAME"
+fi
 
 # Find out the name of the originator of this PR.
 if [ -n "$NAME" ]; then
   ORIGINATOR="$NAME"
 elif [ -f $HOME/.fullname ]; then
   ORIGINATOR="`sed -e '1q' $HOME/.fullname`"
-elif [ -f /bin/domainname ]; then
-  if [ "`/bin/domainname`" != "" -a -f /usr/bin/ypcat ]; then
-    # Must use temp file due to incompatibilities in quoting behavior
-    # and to protect shell metacharacters in the expansion of $LOGNAME
-    /usr/bin/ypcat passwd 2>/dev/null | cat - /etc/passwd | grep "^$LOGNAME:" |
-      cut -f5 -d':' | sed -e 's/,.*//' > $TEMP
-    ORIGINATOR="`cat $TEMP`"
-    rm -f $TEMP
-  fi
-fi
-
-if [ "$ORIGINATOR" = "" ]; then
-  grep "^$LOGNAME:" /etc/passwd | cut -f5 -d':' | sed -e 's/,.*//' > $TEMP
+else
+  # Must use temp file due to incompatibilities in quoting behavior
+  # and to protect shell metacharacters in the expansion of $LOGNAME
+  $PASSWD | grep "^$LOGNAME:" | awk -F: '{print $5}' | sed -e 's/,.*//' > $TEMP
   ORIGINATOR="`cat $TEMP`"
   rm -f $TEMP
 fi
@@ -137,10 +146,12 @@ ARCH=`[ -f /bin/arch ] && /bin/arch`
 MACHINE=`[ -f /bin/machine ] && /bin/machine`
 
 COMMAND=`echo $0 | sed -e 's,.*/,,'`
-USAGE="Usage: $COMMAND [-PVL] [-t address] [-f filename] [--request-id] 
-[--version]"
+USAGE="Usage: $COMMAND [-PVL] [-t address] [-f filename] [-s severity]
+       [-c address] [--request-id] [--version]"
 REMOVE=
 BATCH=
+CC=
+SEVERITY_C=
 
 while [ $# -gt 0 ]; do
   case "$1" in
@@ -157,6 +168,12 @@ while [ $# -gt 0 ]; do
 	fi
 	;;
     -b | --batch) BATCH=true ;;
+    -c | --cc) if [ $# -eq 1 ]; then echo "$USAGE"; exit 1; fi
+	shift ; CC="$1"
+	;;
+    -s | --severity) if [ $# -eq 1 ]; then echo "$USAGE"; exit 1; fi
+	shift ; SEVERITY_C="$1"
+	;;
     -p | -P | --print) PRINT=true ;;
     -L | --list) FORMAT=norm ;;
     -l | -CL | --lisp) FORMAT=lisp ;;
@@ -180,7 +197,7 @@ while [ $# -gt 0 ]; do
  shift
 done
 
-if [ -n "$USER_GNATS_SITE" ]; then
+if [ -n "$USER_GNATS_SITE" ] && [ "$USER_GNATS_SITE" != "$GNATS_SITE" ]; then
   GNATS_SITE=$USER_GNATS_SITE
   GNATS_ADDR=$USER_GNATS_SITE-gnats
 fi
@@ -235,7 +252,9 @@ ORIGINATOR_C='<name of the PR author (one line)>'
 ORGANIZATION_C='<organization of PR author (multiple lines)>'
 CONFIDENTIAL_C='<[ yes | no ] (one line)>'
 SYNOPSIS_C='<synopsis of the problem (one line)>'
-SEVERITY_C='<[ non-critical | serious | critical ] (one line)>'
+if [ -z "$SEVERITY_C" ]; then
+  SEVERITY_C='<[ non-critical | serious | critical ] (one line)>'
+fi
 PRIORITY_C='<[ low | medium | high ] (one line)>'
 CATEGORY_C='<name of the product (one line)>'
 CLASS_C='<[ sw-bug | doc-bug | change-request | support ] (one line)>'
@@ -311,33 +330,24 @@ __EOF__
 To: $GNATS_ADDR
 Subject: 
 From: $FROM
-Reply-To: $REPLY_TO
+Reply-To: $REPLYTO
+Cc: $CC
 X-send-pr-version: $VERSION
 
 
->Submitter-Id:   $SUBMITTER
->Originator: 	 $ORIGINATOR
+>Submitter-Id:	$SUBMITTER
+>Originator:	$ORIGINATOR
 >Organization:
-`
-  if [ -n "$ORGANIZATION" ]; then
-    echo "$ORGANIZATION"
-  else
-    echo "	$ORGANIZATION_C" ;
-  fi ;
-`
->Confidential:  $CONFIDENTIAL_C
+${ORGANIZATION-	$ORGANIZATION_C}
+>Confidential:	$CONFIDENTIAL_C
 >Synopsis:	$SYNOPSIS_C
 >Severity:	$SEVERITY_C
 >Priority:	$PRIORITY_C
->Category: 	$CATEGORY_C
+>Category:	$CATEGORY_C
 >Class:		$CLASS_C
->Release:	 `if [ -n "$DEFAULT_RELEASE" ]; then
-		   echo "$DEFAULT_RELEASE"
-		  else
-		   echo "	$RELEASE_C"
-		  fi; `
+>Release:	${DEFAULT_RELEASE-$RELEASE_C}
 >Environment:
-    	$ENVIRONMENT_C
+	$ENVIRONMENT_C
 `[ -n "$SYSTEM" ] && echo System: $SYSTEM`
 `[ -n "$ARCH" ] && echo Architecture: $ARCH`
 `[ -n "$MACHINE" ] && echo Machine: $MACHINE`
